﻿using System;
//using System.Collections.Generic;
//using System.Linq;
//using System.Text;

namespace TwoCardPokerLib
{
  // -------------------------------------------------
  public class Card
  {
    private string rank;
    private string suit;
    public Card()
    {
      this.rank = "A"; // A, 2, 3, . . ,9, T, J, Q, K
      this.suit = "c"; // c, d, h, s
    }
    public Card(string c)
    {
      this.rank = c[0].ToString();
      this.suit = c[1].ToString();
    }
    public Card(int c) // 0 = Ac, 1 = Ad, . . . , 51 = Ks
    {
      int r = (c % 13) + 1;
      if (r == 1) this.rank = "A";
      else if (r == 10) this.rank = "T";
      else if (r == 11) this.rank = "J";
      else if (r == 12) this.rank = "Q";
      else if (r == 13) this.rank = "K";
      else this.rank = r.ToString(); // 2 to 9

      if (c >= 0 && c <= 12) this.suit = "c";
      else if (c >= 13 && c <= 25) this.suit = "d";
      else if (c >= 26 && c <= 38) this.suit = "h";
      else if (c >= 39 && c <= 51) this.suit = "s";
    }

    public override string ToString()
    {
      return this.rank + this.suit;
    }
    public string Rank
    {
      get { return this.rank; }
    }
    public string Suit
    {
      get { return this.suit; }
    }
    public static bool Beats(Card c1, Card c2)
    {
      if (c1.rank == c2.rank) // short-circuit out on a tie
        return false;

      if (c1.rank == "A")
      {
        if (c2.rank != "A")
          return true;
        else
          return false;
      }
      else if (c1.rank == "K")
      {
        if (c2.rank == "A" || c2.rank == "K") // redundant check here . . . 
          return false;
        else
          return true;
      }
      else if (c1.rank == "Q")
      {
        if (c2.rank == "A" || c2.rank == "K" || c2.rank == "Q")
          return false;
        else
          return true;
      }
      else if (c1.rank == "J")
      {
        if (c2.rank == "A" || c2.rank == "K" || c2.rank == "Q" || c2.rank == "J")
          return false;
        else
          return true;
      }
      else if (c1.rank == "T")
      {
        if (c2.rank == "A" || c2.rank == "K" || c2.rank == "Q" || c2.rank == "J" || c2.rank == "T")
          return false;
        else
          return true;
      }
      else
      { // c1.rank is 2, 3, . . 9. c2.rank is either A-T or 2-9
        if (c2.rank == "A" || c2.rank == "K" || c2.rank == "Q" || c2.rank == "J" || c2.rank == "T")
          return false;
        else // both c1.rank and c2.rank are 2-9
        {
          int c1Rank = int.Parse(c1.rank);
          int c2Rank = int.Parse(c2.rank);
          return c1Rank > c2Rank;
        }
      }
    } // Beats()

    public static bool Ties(Card c1, Card c2)
    {
      return (c1.rank == c2.rank);
    }
  } // class Card

  // -------------------------------------------------
  public class Deck
  {
    private Card[] cards;
    private int top;
    private Random random = null;

    public Deck()
    {
      this.cards = new Card[52];
      for (int i = 0; i < 52; ++i)
        this.cards[i] = new Card(i);
      this.top = 0;
      random = new Random(0);
    }

    public void Shuffle()
    {
      for (int i = 0; i < 52; ++i)
      {
        int r = random.Next(i, 52);
        Card temp = this.cards[r];
        this.cards[r] = this.cards[i];
        this.cards[i] = temp;
      }
      this.top = 0;
    }

    public int Count()
    {
      return 52 - this.top;
    }

    public override string ToString()
    {
      string s = "";
      for (int i = this.top; i < 52; ++i)
      {
        s += "[ " + i + " ] " + this.cards[i].ToString() + "\n";
      }
      return s;
    }

    public Card[] Deal(int n)
    {
      //try
      //{
        Card[] result = new Card[n];
        for (int i = 0; i < n; ++i)
        {
          result[i] = this.cards[this.top + i];
        }
        this.top += n;

        return result;
      //}
      //catch (IndexOutOfRangeException)
      //{
      //  return new Card[2];
      //}
    }

  } // Deck

  // -------------------------------------------------
  public class Hand
  {
    private Card card1; // high card
    private Card card2; // low card
    public Hand()
    {
      this.card1 = new Card(); // high card?
      this.card2 = new Card(); // low card?

      if (Card.Beats(card2, card1))
      {
        Card temp = this.card1;
        this.card1 = this.card2;
        this.card2 = temp;
        //Card temp = new Card(this.card1.ToString());
        //this.card1 = new Card(this.card2.ToString());
        //this.card2 = new Card(temp.ToString());

      }
    }
    public Hand(Card c1, Card c2)
    {
      this.card1 = new Card(c1.Rank + c1.Suit);
      this.card2 = new Card(c2.Rank + c2.Suit);
      if (Card.Beats(card2, card1))
      {
        Card temp = this.card1;
        this.card1 = this.card2;
        this.card2 = temp;

      }
    }
    public Hand(string s1, string s2)
    {
      this.card1 = new Card(s1);
      this.card2 = new Card(s2);
      if (Card.Beats(card2, card1))
      {
        Card temp = this.card1;
        this.card1 = this.card2;
        this.card2 = temp;

      }
    }
    public override string ToString()
    {
      return this.card1.ToString() + "-" + this.card2.ToString();
    }

    private bool IsPair() { return this.card1.Rank == this.card2.Rank; }
    private bool IsFlush() { return this.card1.Suit == this.card2.Suit; }
    private bool IsStraight()
    {
      if (this.card1.Rank == "A" && this.card2.Rank == "K") return true; // boundary
      else if (this.card1.Rank == "K" && this.card2.Rank == "Q") return true;
      else if (this.card1.Rank == "Q" && this.card2.Rank == "J") return true;
      else if (this.card1.Rank == "J" && this.card2.Rank == "T") return true;
      else if (this.card1.Rank == "T" && this.card2.Rank == "9") return true;
      else if (this.card1.Rank == "9" && this.card2.Rank == "8") return true;
      else if (this.card1.Rank == "8" && this.card2.Rank == "7") return true;
      else if (this.card1.Rank == "7" && this.card2.Rank == "6") return true;
      else if (this.card1.Rank == "6" && this.card2.Rank == "5") return true;
      else if (this.card1.Rank == "5" && this.card2.Rank == "4") return true;
      else if (this.card1.Rank == "4" && this.card2.Rank == "3") return true;
      else if (this.card1.Rank == "3" && this.card2.Rank == "2") return true;
      else if (this.card1.Rank == "A" && this.card2.Rank == "2") return true; // boundary
      else return false;
    }
    private bool IsStraightFlush() { return this.IsStraight() && this.IsFlush(); }

    private bool Beats(Hand h) // is this > h??
    {
      // Straight Flush > Pair > Flush > Straight > Ace high > King high > . . > Four high
      if (this.IsStraightFlush())
      {
        if (!h.IsStraightFlush()) return true; // this is a straight flush and h isn't
        if (h.IsStraightFlush()) // both hands are straight flushes . .
        {
          if (Card.Beats(this.card1, h.card1)) // this is a higher straight flush
            return true;
          else
            return false;
        }
      } // this.IsStraightFlush()
      else if (this.IsPair())
      {
        if (h.IsStraightFlush()) return false; // h has a straight flush
        if (!h.IsPair()) return true;
        if (h.IsPair()) // both hands are pairs
        {
          if (Card.Beats(this.card1, h.card1)) // this is a higher pair
            return true;
          else
            return false; // this is a lower or equal pair
        }
      } // this.IsPair
      else if (this.IsFlush())
      {
        if (h.IsStraightFlush()) return false; // h has a straight flush
        if (h.IsPair()) return false; // h has a pair
        if (!h.IsFlush()) return true;
        if (h.IsFlush()) // both hands are flushes
        {
          if (Card.Beats(this.card1, h.card1)) // this is a higher flush on first card
            return true;
          else if (Card.Ties(this.card1, this.card1) && Card.Beats(this.card2, h.card2)) // this is a higher flush on 2nd card
            return true;
          else
            return false; // this is a lower or equal flush
        }
      } // this.IsFlush
      else if (this.IsStraight())
      {
        if (h.IsStraightFlush()) return false; // h has a straight flush
        if (h.IsPair()) return false; // h has a pair
        if (h.IsFlush()) return false; // h has a flush
        if (!h.IsStraight()) return true;
        if (h.IsStraight()) // both hands are straights
        {
          if (Card.Beats(this.card1, h.card1)) // this is a higher straight
            return true;
          else
            return false; // this is a lower or equal straight
        }
      } // this.IsStraight
      else // this is Ace-high down to Four-high ( Three-high and Two-high impossible)
      {
        if (Card.Beats(this.card1, h.card1))
          return true;
        else if (Card.Beats(h.card1, this.card1))
          return false;
        else if (Card.Ties(this.card1, h.card1)) // this.card1 == h.card1
        {
          if (Card.Beats(this.card2, h.card2))
            return true;
          else if (Card.Beats(h.card2, this.card2))
            return false;
        }
      } // this is Ace-high down to Four-high
      return false;
    } // Hand.Beats

    private bool Ties(Hand h) // does this tie h?
    {
      if (this.IsStraightFlush() && h.IsStraightFlush() && this.card1.Rank == h.card1.Rank)
        return true;
      else if (this.IsPair() && h.IsPair() && this.card1.Rank == h.card1.Rank)
        return true;
      else if (this.IsFlush() && h.IsFlush() && this.card1.Rank == h.card1.Rank) // deliberate error; must check 2nd card too
        return true;
      //else if (this.IsFlush() && h.IsFlush() && this.card1.Rank == h.card1.Rank && this.card2.Rank == h.card2.Rank)
      //  return true;
      else if (this.IsStraight() && h.IsStraight() && this.card1.Rank == h.card1.Rank)
        return true;
      else if (this.card1.Rank == h.card1.Rank && this.card2.Rank == h.card2.Rank)
        return true;
      else
        return false;
    } // Hand.Ties

    public int Compare(Hand h)
    {
      if (this.Beats(h))
        return 1;
      else if (this.Ties(h))
        return 0;
      else if (h.Beats(this))
        return -1;
      else
        throw new Exception("Illegal path in Compare()");
    }

    public enum HandType
    {
      StraightFlush, Pair, Flush, Straight, AceHigh, KingHigh, QueenHigh, TenHigh,
      NineHigh, EightHigh, SevenHigh, SixHigh, FiveHigh, FourHigh, Invalid
    }

  } // class Hand


} // ns TwoCardPokerLib
